# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from config_tempest import constants as C
from config_tempest.services.base import Service
from tempest.lib import exceptions


class MeteringService(Service):

    def post_configuration(self, conf, is_service):
        try:
            params = {'type': 'metering'}
            services = self.client.list_services(**params)
        except exceptions.Forbidden:
            C.LOG.warning("User has no permissions to list services - "
                          "metering service can't be discovered.")
            return
        if services and len(services['services']):
            metering = services['services'][0]
            if 'ceilometer' in metering['name'] and metering['enabled']:
                conf.set('service_available', 'ceilometer', 'True')

    @staticmethod
    def get_service_type():
        return ['metering']

    @staticmethod
    def get_codename():
        return 'ceilometer'
