# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from config_tempest.services.base import Service


class Ec2Service(Service):

    def set_default_tempest_options(self, conf):
        conf.set('aws', 'ec2_url', self.service_url)

    @staticmethod
    def get_service_type():
        return ['ec2']


class S3Service(Service):

    def set_default_tempest_options(self, conf):
        conf.set('aws', 's3_url', self.service_url)

    @staticmethod
    def get_service_type():
        return ['s3']
