# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import re
from six.moves import urllib


def get_base_url(endpoint):
    """Return the base url.

    :param endpoint: endpoint
    :type endpoint: string
    :returns: base_url - without version
    :rtype: string
    """
    url = urllib.parse.urlsplit(endpoint)
    # the regex matches a version in the url path
    url_path_without_version = re.split(r'(^|/)+v\d+(\.\d+)?', url.path)[0]
    url = list(url)
    url[2] = url_path_without_version + '/'
    return urllib.parse.urlunsplit(url)
